package shukan;


public class ShukanData
{
  /** Displayed calendar */
  private ShukanSemester sem = null;
  /** Cursus */
  private String[] curs = null;
  /** Modules */
  private ShukanModule[][] mod = null;
  /** Teachers */
  private String[] teachers = new String[0];
  /** Sort for teachers */
  private int[] tSort = new int[0];
  /** Students */
  private String[] students = {"Etu"};
  /** Sort for students */
  private int[] sSort = {0};
  /** Scolar year (for displays) */
  private int scolarYear = 0;

  /** Selected cursus */
  private int selCur = 0; // First by default
  /** Selected module */
  private int selMod = -1; // None by default
  /** Selected load */
  private int selLoad = 0; // Student by default
  /** Selection first week */
  private int selWeek = -1; // None by default
  /** Selected first week and activity */
  private int selAct = -1;
  /** Selection length */
  private int selLength = 0;


  /** Creates a empty data base */
  public void setSemester (int scolYear, ShukanSemester sem)
  {
    this.scolarYear = scolYear;
    this.sem = sem;
  }

  /** Returns the number of weeks in the semester */
  public int semesterSize ()
  {
    return (sem.size ());
  }

  /** Returns the week numbers */
  public int[] weekNumbers ()
  {
    return (sem.weekNumbers ());
  }

  /** Returns the start week number */
  public int startWeekNumber ()
  {
    return (sem.startWeekNumber ());
  }

  /** Returns the last week of year number */
  public int lastWeekOfYearNumber ()
  {
    return (sem.lastWeekOfYearNumber ());
  }

  /** Returns the standard week duration */
  public int standardWeekDuration ()
  {
    return (sem.standardWeekDuration ());
  }

  /** Returns the week durations */
  public int[] weekDurations ()
  {
    return (sem.weekDurations ());
  }

  /** Creates the modules array */
  public void declareCursus (String[] names)
  {
    curs = names;
    mod = new ShukanModule[names.length][];
  }

  /** Creates the modules array */
  public void setModules (int num, String[] names)
  {
    mod[num] = new ShukanModule[names.length];
    for (int i = 0; i < names.length; i++)
    {
      mod[num][i] = new ShukanModule (names[i]);
      mod[num][i].setActiveWeeks (sem.copyWeekDurations ());
    }
  }

  /** Adds an activity to a module */
  public boolean addActivity (int cursNum, int modNum, int actNum, String type)
  {
    return (mod[cursNum][modNum].addActivity (actNum, type));
  }

  /** Closes the activities of a module */
  public void closeActivities (int cursNum, int modNum, int size)
  {
    mod[cursNum][modNum].closeActivities (size);
  }

  /** Automatically plans a free module */
  public void autoPlan (int cursNum, int modNum)
  {
    if (! mod[cursNum][modNum].isScheduled ())
      mod[cursNum][modNum].autoPlan ();
  }

  /** Schedules the activities of a module */
  public void setSchedule (int cursNum, int modNum, int[] weeks)
  {
    mod[cursNum][modNum].schedule (weeks);
  }

  /** Unschedules the activities of a module */
  public void unscheduleActivities (int cursNum, int modNum)
  {
    mod[cursNum][modNum].unschedule ();
  }

  /** Schedules an activity of a module */
  public void scheduleActivity (int cursNum, int modNum, int activity, int week)
  {
    mod[cursNum][modNum].schedule (activity, week);
  }

  /** Returns the number of modules */
  public int numberOfModules ()
  {
    return (mod[selCur].length);
  }

  /** Returns the name of the active cursus */
  public String cursusName ()
  {
    return (curs[selCur]);
  }
  /** Returns the number of a cursus */
  public int cursusNumber(){
    return selCur;
  }

  /** Returns the name of a cursus */
  public String cursusName (int num)
  {
    return (curs[num]);
  }

  /** Returns the names of the modules */
  public String[] moduleNames ()
  {
    String[] names = new String[mod[selCur].length];
    for (int i = 0; i < mod[selCur].length; i++)
      names[i] = mod[selCur][i].name ();
    return (names);
  }

  /** Returns the names of the modules */
  public String[] moduleNames (int cur)
  {
    String[] names = new String[mod[cur].length];
    for (int i = 0; i < mod[cur].length; i++)
      names[i] = mod[cur][i].name ();
    return (names);
  }

  /** Sets a module start week */
  public void setModuleStart (int cursNum, int modNum, int week)
  {
    mod[cursNum][modNum].setStartWeek (sem.weekIndex (week));
  }

  /** Sets a module end week */
  public void setModuleEnd (int cursNum, int modNum, int week)
  {
    mod[cursNum][modNum].setEndWeek (sem.weekIndex (week));
  }

  /** Sets a module free week */
  public void setModuleHolly (int cursNum, int modNum, int week)
  {
    mod[cursNum][modNum].setHollyWeek (sem.weekIndex (week));
  }

  /** Returns the active weeks of a module */
  public int[] activeWeeks (int modNum)
  {
    return (mod[selCur][modNum].activeWeeks ());
  }

  /** Returns the activities of a module */
  public int[] activities (int modNum)
  {
    return (mod[selCur][modNum].activities ());
  }

  /** Returns the activity schedule of a module */
  public int[] scheduleInWeek (int modNum, int week)
  {
    return (mod[selCur][modNum].schedule (week));
  }

  /** Returns the whole schedule of a module */
  public int[] schedule (int modNum)
  {
    return (mod[selCur][modNum].schedule ());
  }

  /** Returns the whole schedule of a module */
  public int[] schedule (int cur, int modNum)
  {
    return (mod[cur][modNum].schedule ());
  }

  /** Returns the selected module */
  public int selectedModule ()
  {
    return (selMod);
  }

  /** Returns the first selected week */
  public int selectedWeek ()
  {
    return (selWeek);
  }

  /** Returns the selection length */
  public int selectedLength ()
  {
    return (selLength);
  }

  /** Deactivates the module and week */
  public void deactivate ()
  {
    selMod = -1;
    selWeek = -1;
    selLength = 0;
  }

  /** Activates a whole module */
  public boolean activate (int modNum)
  {
    selMod = modNum;
    selWeek = 0;
    selLength = semesterSize ();
    return (true);
  }

  /** Activates a module at given week */
  public boolean activate (int modNum, int week, boolean zone)
  {
    if (zone)
    {
      if (selMod != modNum) return (false);
      if (selWeek == -1) return (false);
      if (selLength <= 2 && week >= selWeek && week <= selWeek + selLength - 1)
        return (false);
      if (week < selWeek)
      {
        selLength += selWeek - week;
        selWeek = week;
        return (true);
      }
      if (week >= selWeek + selLength)
      {
        selLength = week - selWeek + 1;
        return (true);
      }
      int dist = week - selWeek;
      if (2 * dist <= selLength - 1)
      {
        selLength -= week - selWeek;
        selWeek = week;
      }
      else selLength = week - selWeek + 1;
      return (true);
    }
    else
    {
      selMod = modNum;
      selWeek = week;
      selLength = 1;
    }
    return (true);
  }

  /** Activates a whole module */
  public boolean activateAct (int modNum)
  {
    System.out.println ("Activation  du module " + modNum);
    return (false);
/*
    selMod = modNum;
    selWeek = 0;
    selLength = semesterSize ();
    return (true);
*/
  }

  /** Shifts the selection leftwards */
  public boolean left ()
  {
    if (selMod == - 1 || selLength == 0 || selWeek == -1) return (false);
    mod[selCur][selMod].left (selWeek, selLength);
    return (true);
  }

  /** Shifts the selection rightwards */
  public boolean right ()
  {
    if (selMod == - 1 || selLength == 0 || selWeek == -1) return (false);
    mod[selCur][selMod].right (selWeek, selLength);
    return (true);
  }

  /** Shifts the selection leftwards and follows it */
  public boolean followLeft ()
  {
    if (selMod == - 1 || selLength == 0 || selWeek == -1) return (false);
    selWeek = mod[selCur][selMod].left (selWeek, selLength);
    return (true);
  }

  /** Shifts the selection rightwards and follows it */
  public boolean followRight ()
  {
    if (selMod == - 1 || selLength == 0 || selWeek == -1) return (false);
    selWeek = mod[selCur][selMod].right (selWeek, selLength);
    return (true);
  }

  /** Returns the load affected to selected agent for given week */
  public int computeLoad (int week)
  {
    int load = 0;
    if (selLoad < 0)
      for (int j = 0; j < curs.length; j++)
        for (int i = 0; i < mod[j].length; i++)
          load += mod[j][i].teacherLoad (tSort[selLoad + teachers.length],
                                         week);
    else for (int i = 0; i < mod[selCur].length; i++)
      load += mod[selCur][i].studentLoad (week);
    return (load);
  }

  /** Returns the load affected to standard student for given week */
  public int studentLoad (int week)
  {
    int load = 0;
    for (int i = 0; i < mod[selCur].length; i++)
      load += mod[selCur][i].studentLoad (week);
    return (load);
  }

  /** Affects activities to teachers */
  public void affectTo (int curs, int mod, int typAct, String[] names)
  {
    int[] numAff = new int[names.length];
    if (teachers.length == 0)
    {
      teachers = new String[] {names[0]};
      tSort = new int[] {0};
    }
    for (int i = 0; i < names.length; i++)
      numAff[i] = findTeacher (names[i], 0, tSort.length);
    this.mod[curs][mod].setTeachers (typAct, numAff);
  }

  /** Affects specific activity to teachers */
  public void affectSpec (int curs, int mod, int numAct, String[] names)
  {
    int[] numAff = new int[names.length];
    if (teachers.length == 0)
    {
      teachers = new String[] {names[0]};
      tSort = new int[] {0};
    }
    for (int i = 0; i < names.length; i++)
      numAff[i] = findTeacher (names[i], 0, tSort.length);
    this.mod[curs][mod].affectSpec (numAct, numAff);
  }

  /** Finds the rank of a teacher */
  private int findTeacher (String name, int from, int to)
  {
    int test = (from + to) / 2;
    if (test < tSort.length
        && name.equals (teachers[tSort[test]])) return tSort[test];
    if (to - from < 1)
    {
      String[] tmp = new String[teachers.length + 1];
      for (int i = 0; i < teachers.length; i ++) tmp[i] = teachers[i];
      tmp[teachers.length] = name;
      int[] ts = new int[tmp.length];
      for (int i = 0; i < to; i ++) ts[i] = tSort[i];
      ts[to] = teachers.length;
      for (int i = to; i < teachers.length; i ++) ts[i+1] = tSort[i];
      teachers = tmp;
      tSort = ts;
      return (tSort[to]);
    }
    if (name.compareTo (teachers[tSort[test]]) > 0)
      return (findTeacher (name, test + 1, to));
    return (findTeacher (name, 0, test));
  }

  /** Selects the next load */
  public void toggleLoad (int direction)
  {
    selLoad += direction;
    if (selLoad >= students.length) selLoad = - teachers.length;
    else if (selLoad < - teachers.length) selLoad = students.length - 1;
  }

  /** Returns the selected load name */
  public String loadName ()
  {
    if (selLoad < 0) return (teachers[tSort[selLoad + teachers.length]]);
    return (students[sSort[selLoad]]);
  }

  /** Sets an activity to be displayed on left */
  public void setOnLeft (int curs, int mod, int act)
  {
    this.mod[curs][mod].setOnLeft (act);
  }

  /** Sets an activity to be displayed on right */
  public void setOnRight (int curs, int mod, int act)
  {
    this.mod[curs][mod].setOnRight (act);
  }

  /** Returns activities to be displayed on left */
  public boolean[] onLeft (int mod)
  {
    return (this.mod[selCur][mod].activitiesOnLeft ());
  }

  /** Returns activities to be displayed on right */
  public boolean[] onRight (int mod)
  {
    return (this.mod[selCur][mod].activitiesOnRight ());
  }

  /** Returns the number of cursus */
  public int countOfCursus ()
  {
    return (mod.length);
  }

  /** Toggles the selected cursus */
  public void toggleCursus (boolean on)
  {
    if (on)
    {
      if (++selCur == mod.length) selCur = 0;
    }
    else if (--selCur == -1) selCur = mod.length - 1;
    selMod = -1;
  }

  /** Sets a subtitle to given module */
  public void setSubtitle (int curs, int mod, String text)
  {
    this.mod[curs][mod].setSubtitle (text);
  }

  /** Returns the subtitle of selected module */
  public String subtitle (int module)
  {
    return (mod[selCur][module].subtitle ());
  }

  /** Returns the subtitle of selected module */
  public String subtitle (int curs, int module)
  {
    return (mod[curs][module].subtitle ());
  }

  /** Returns the first scolar year */
  public int scolarYear ()
  {
    return (scolarYear);
  }

  /** returns the teachers array */
  public String[] teachers ()
  {
    return (teachers);
  }

  /** Returns a teacher name */
  public String teacherName (int num)
  {
    return (teachers[tSort[num]]);
  }

  /** Returns a teacher load on given week */
  public int teacherLoad (int num, int week)
  {
    int load = 0;
    for (int j = 0; j < curs.length; j++)
      for (int i = 0; i < mod[j].length; i++)
        load += mod[j][i].teacherLoad (tSort[num], week);
    return (load);
  }

  /** Returns the progression of given module */
  public String[] progression (int curs, int mod)
  {
    return (this.mod[curs][mod].progression ());
  }

  /** Returns the count of actvities in given module */
  public int activityCount (int curs, int mod)
  {
    return (this.mod[curs][mod].activityCount ());
  }

  /** Returns the start week of given module */
  public int moduleStart (int curs, int mod)
  {
    int[] wk = this.mod[curs][mod].activeWeeks ();
    if (wk[0] != 0) return (0);
    int sw = 1;
    while (wk[sw] == 0)
      if (++sw == wk.length) return (0);
    sw += sem.startWeekNumber ();
    if (sw > sem.lastWeekOfYearNumber ()) sw -= sem.lastWeekOfYearNumber ();
    return (sw);
  }

  /** Returns the end week of given module */
  public int moduleEnd (int curs, int mod)
  {
    int[] wk = this.mod[curs][mod].activeWeeks ();
    if (wk[wk.length - 1] != 0) return (0);
    int ew = wk.length - 2;
    while (wk[ew] == 0)
      if (--ew == -1) return (0);
    ew += sem.startWeekNumber ();
    if (ew > sem.lastWeekOfYearNumber ()) ew -= sem.lastWeekOfYearNumber ();
    return (ew);
  }

  /** Returns the end week of given module */
  public int[] moduleHollyWeeks (int curs, int mod)
  {
    int[] wk = this.mod[curs][mod].activeWeeks ();
    int lw = wk.length - 1;
    while (wk[lw] == 0)
      if (--lw == -1) return (new int[0]);

    int[] sk = sem.weekDurations ();
    int[] hk = new int[sk.length];
    int nb = 0;
    boolean ok = false;
    for (int i = 0; i < lw; i++)
    {
      if (wk[i] != 0) ok = true;
      else if (ok && sk[i] != 0) hk[nb++] = i;
    }

    int[] res = new int[nb];
    int sw = sem.startWeekNumber ();
    int lyw = sem.lastWeekOfYearNumber ();
    for (int i = 0; i < nb; i++)
    {
      res[i] = hk[i] + sw;
      if (res[i] > lyw) res[i] -= lyw;
    }
    return (res);
  }

  /** Returns the teachers of given module */
  public String[] moduleTeachers (int curs, int mod, int typact)
  {
    int[] ids = this.mod[curs][mod].teachers (typact);
    if (ids == null) return (new String[0]);
    String[] tch = new String[ids.length];
    for (int i = 0; i < tch.length; i++) tch[i] = teachers[ids[i]];
    return (tch);
  }

  /** Returns the specific teachers of given module */
  public String[][] moduleSpecTeachers (int curs, int mod)
  {
    int[][] ids = this.mod[curs][mod].specTeachers ();
    String[][] tch = new String[ids.length][];
    for (int i = 0; i < ids.length; i++)
    {
      tch[i] = new String[ids[i].length];
      for (int j = 0; j < ids[i].length; j++) tch[i][j] = teachers[ids[i][j]];
    }
    return (tch);
  }

  /** Returns the set-on-left activities of given module */
  public boolean[] moduleOnLeft (int curs, int mod)
  {
    return (this.mod[curs][mod].onLeftActivities ());
  }

  /** Returns the set-on-right activities of given module */
  public boolean[] moduleOnRight (int curs, int mod)
  {
    return (this.mod[curs][mod].onRightActivities ());
  }

  /** Returns the formatted date */
  public String date ()
  {
    return (sem.date ());
  }

  /** Returns the semester */
  public ShukanSemester semester ()
  {
    return (sem);
  }

  /** Selects an activity */
  public boolean select (int mod, int pos, boolean zone)
  {
    int selection = this.mod[selCur][mod].select (pos);
    if (selection != -1)
    {
      if (selWeek != -1) deactivate ();

      if (zone && selMod == mod && selection != selAct)
      {
        if (selection < selAct)
        {
          selLength += selAct - selection;
          selAct = selection;
        }
        else if (selection < selAct + selLength)
        {
          if (selection - selAct < selLength / 2)
          {
            selLength -= (selection - selAct);
            selAct = selection;
          }
          else
            selLength = selection - selAct + 1;
        }
        else
          selLength = selection - selAct + 1;
      }
      else
      {
        selMod = mod;
        selLength = 1;
        selAct = selection;
      }
      return true;
    }
    return false;
  }

  /** Returns the first selected activity */
  public int selectedActivity ()
  {
    return (selAct);
  }

  /** Exchanges extreme activities of the selected area */
  public void exchangeExtremes ()
  {
    if (selMod != -1 && selAct != -1 && selLength > 1)
      this.mod[selCur][selMod].exchange (selAct, selAct + selLength - 1);
  }

  /** test if a week is active for a module */
  public boolean isActiveWeek(int week, int mod){
    int debut = moduleStart(cursusNumber(), mod);
    if (week>=debut && (week<=moduleEnd(cursusNumber(), mod) || moduleEnd(cursusNumber(), mod)==0))
    {
      int[] holly_weeks = moduleHollyWeeks(cursusNumber(), mod);
      int[] week_durations = weekDurations();
      // periode d'indisponibilité
      for (int i =0; i<holly_weeks.length; i++) {
        if (week == holly_weeks[i]) return false;
      }
      // hollydays
      if (debut == 0) { if (week_durations[week-debut-4]==0) return false; }
      else { if (week_durations[week-debut]==0) return false; }

      return true;
    }
    // avant début de module ou après fin de module
    return false;
  }
}
